@@ -1,4 +1,3 @@
-from abc import *
 import os
 import sys
 import json
@@ -13,6 +12,9 @@ from sklearn.model_selection import train_test_split
 from .. import global_variable as gv
 from . import dataset_mgmt as dsmgmt
 
+import pandas as pd
+
+
 class MiddleDatasetMgmt(dsmgmt.DatasetMgmt):
     
     def __init__(self, global_variable, log):
@@ -37,29 +39,34 @@ class MiddleDatasetMgmt(dsmgmt.DatasetMgmt):
         return tf.train.Feature(int64_list=tf.train.Int64List(value=[value]))
     
     
-    def _make_example(self, image_bytes, label_bytes):
+    def _make_example(self, image_bytes, label_bytes): # write
         feature = {'image': self._bytes_feature(image_bytes),
                    'segmentation_mask': self._bytes_feature(label_bytes)}
         return tf.train.Example(features=tf.train.Features(feature=feature))
     
     
     def _get_dataset_metadata(self, bucket):
-        blob = bucket.blob(self.gv.DATASET_METADATA_NAME) #metadata.json    
+        blob = bucket.blob(self.gv.DEV_DATASET_METADATA_NAME) if self.gv.DEV_MODE else bucket.blob(self.gv.DATASET_METADATA_NAME)
+        
+        if self.gv.DEV_MODE and None is blob:
+            print(f"{self.gv.DEV_DATASET_METADATA_NAME} is not exist")
+            blob = bucket.blob(self.gv.DATASET_METADATA_NAME)
+        
         with blob.open("r") as f:
             metadata_list = json.loads(f.read())
+            
         return metadata_list
     
     
-    def _parse_image_function(self, example_proto):   
-        return tf.io.parse_single_example(example_proto, self._image_feature_description)
+    def _parse_image_function(self, example_proto): # read
+        image_feature_description = {
+            'image': tf.io.FixedLenFeature([], tf.string),
+            'segmentation_mask':tf.io.FixedLenFeature([], tf.string),
+        }
+        return tf.io.parse_single_example(example_proto, image_feature_description)
     
     
-    _image_feature_description = {
-        'image': tf.io.FixedLenFeature([], tf.string),
-        'segmentation_mask':tf.io.FixedLenFeature([], tf.string),
-    }
-    
-    def _augmentation(self, image, segmentation_mask):
+    def _augmentation(self, image, segmentation_mask, augmente_type):
         image_aug = tf.image.flip_left_right(image)
         segmentation_mask_aug = tf.image.flip_left_right(segmentation_mask)
         return image_aug, segmentation_mask_aug
@@ -97,15 +104,25 @@ class MiddleDatasetMgmt(dsmgmt.DatasetMgmt):
         self.log.info("under contruction")
     
     
+    def create_dev_dataset_metadata(self, bucket_name, start_idx, end_idx):
+        bucket = self.client.get_bucket(bucket_name)
+        metadata_list = self._get_dataset_metadata(bucket)
+        
+        blob = Blob(self.gv.DEV_DATASET_METADATA_NAME, self.client.get_bucket(bucket_name))
+        blob.upload_from_string(json.dumps(metadata_list[start_idx:end_idx]))
+        
+        
     #dataset_begin_idx는 raw dataset source가 다른 것을 적용하는 시점에는 반듯이 삭제해야 함 
-    def read_and_split(self, bucket_name, dataset_begin_idx, dataset_size, data_split): #TFRecord format : storing a sequence of binary records
+    def read_and_split(self, bucket_name, dataset_bucket_idx, data_split): #TFRecord format : storing a sequence of binary records
         test_set = []
         train_val_set = []
         
         bucket = self.client.get_bucket(bucket_name)
         metadata_list = self._get_dataset_metadata(bucket)
         
-        for metadata in metadata_list[dataset_begin_idx:dataset_begin_idx+dataset_size]:
+        print(f"bucket_name={bucket_name}")
+        
+        for metadata in metadata_list:
             blob = bucket.blob(metadata[self.gv.DATASET_METADATA_KEY_SOURCE_REF]) 
             with blob.open("rb") as f:
                 image_bytes = f.read()
@@ -136,14 +153,11 @@ class MiddleDatasetMgmt(dsmgmt.DatasetMgmt):
     def read_images(self, path):  #TFRecord format
         image_dataset = tf.data.TFRecordDataset(path, self.gv.DATASET_COMPRESS_TYPE)
         assert isinstance(image_dataset, tf.data.Dataset)
-        parsed_image_dataset = image_dataset.map(self._parse_image_function)
+        parsed_image_dataset = image_dataset.map(self._parse_image_function, num_parallel_calls=tf.data.experimental.AUTOTUNE)
         return parsed_image_dataset
-        
-        # with tf.data.TFRecordDataset(path, self.gv.DATASET_COMPRESS_TYPE) as image_dataset:
-        #     return image_dataset.map(_parse_image_function)
                 
         
-    def augmente_image(self, parsed_image_dataset):
+    def augmente_image(self, parsed_image_dataset, augmente_type):
         train_aug_set = []
         for image_features in parsed_image_dataset:
             img = image_features['image']
@@ -151,14 +165,34 @@ class MiddleDatasetMgmt(dsmgmt.DatasetMgmt):
 
             train_aug_set.append(self._make_example(img, segment_img))
 
-            image_aug, segmentation_mask_aug = self._augmentation(tf.io.decode_image(img, channels=3), tf.io.decode_png(segment_img, channels=1))
+            image_aug, segmentation_mask_aug = self._augmentation(tf.io.decode_image(img, channels=3), tf.io.decode_png(segment_img, channels=1), augmente_type)
             train_aug_set.append(self._make_example(tf.io.encode_jpeg(image_aug), tf.io.encode_png(segmentation_mask_aug)))
             
         return train_aug_set
 
 
-                
+    def merge_image_dataset(self, dataset_paths):
+        self.log.info(f"merge dataset {dataset_paths[0]} and {dataset_paths[1]}")
+        
+        dataset1 = self.read_images(dataset_paths[0])
+        print(f"dataset1={dataset1}\n\n")
+        dataset2 = self.read_images(dataset_paths[1])
+        print(f"dataset2={dataset2}\n\n")
+        dataset = dataset1.concatenate(dataset2)
+        print(f"dataset_concatenate={dataset}\n\n")
+        # dataset = pd.merge(dataset, dataset_append)
+        # print(f"dataset_concatenate={dataset}\n\n")
+        
+       
+        data_set = [] 
+        for image in dataset:
+            data_set.append(self._make_example(image['image'], image['segmentation_mask']))
+            
+        return data_set
+       
+        
 
+# Cloud와 독립되도록 보완하자 => Module 분리
 # Download, Upload to GCS
 class TensorflowDatasetMgmt(MiddleDatasetMgmt):
     
@@ -169,17 +203,25 @@ class TensorflowDatasetMgmt(MiddleDatasetMgmt):
     def upload(self, bucket_name):
         self.bucket_name = bucket_name
         
-        dataset_list = self.__upload_dataset_to_bucket(self.dataset['train'], 'train')
-        dataset_list.extend(self.__upload_dataset_to_bucket(self.dataset['test'], 'test'))
-        self.__upload_dataset_label_to_bucket(dataset_list)
+        dataset_label = self.__upload_dataset_to_bucket(self.dataset['train'], 'train')
+        dataset_label.extend(self.__upload_dataset_to_bucket(self.dataset['test'], 'test'))
+        self.__upload_dataset_label_to_bucket(dataset_label)
     
     
+    # {
+    #   "source-ref": "original/train/image/Sphynx_158.jpg",
+    #   "label-ref": "original/train/label/Sphynx_158.png",
+    #   "data-type": "train"
+    # },
     def __upload_dataset_to_bucket(self, dataset, data_type_name):
-        dataset_json = []
+        dataset_label = []
         bucket = self.client.get_bucket(self.bucket_name)
-
+      
         data_iterator = dataset.as_numpy_iterator()
-        for data in data_iterator:
+        for idx, data in enumerate(data_iterator):
+            # if self.gv.DEV_MODE and self.gv.DEV_MODE_DATASET_SIZE == idx:
+            #     break
+            
             f_name = data['file_name'].decode('utf-8')
             image = data['image']
             label = data['segmentation_mask']
@@ -187,19 +229,23 @@ class TensorflowDatasetMgmt(MiddleDatasetMgmt):
             blob = Blob(f"original/{data_type_name}/image/{f_name}", bucket)
             blob.upload_from_file(BytesIO(cv.imencode('.jpg', image)[1]))
 
-            blob = Blob(f"original/{data_type_name}/label/{os.path.splitext(f_name)[0]}.png", bucket) #파일 이름 splitext로 분리
+            blob = Blob(f"original/{data_type_name}/label/{os.path.splitext(f_name)[0]}.png", bucket)
             blob.upload_from_file(BytesIO(cv.imencode('.png', label)[1]))
 
-            dataset_json.append({'source-ref': f"original/{data_type_name}/image/{f_name}", 
+            dataset_label.append({'source-ref': f"original/{data_type_name}/image/{f_name}", 
                                  'label-ref': f"original/{data_type_name}/label/{os.path.splitext(f_name)[0]}.png", 
                                  'data-type': data_type_name})
 
-        return dataset_json
+        return dataset_label
     
     
-    def __upload_dataset_label_to_bucket(self, dataset_list): 
-        blob = Blob(self.gv.DATASET_LABEL_NAME, self.client.get_bucket(self.bucket_name))
-        blob.upload_from_string(json.dumps(dataset_list))
+    def __upload_dataset_label_to_bucket(self, dataset_label): 
+        blob = Blob(self.gv.DATASET_METADATA_NAME, self.client.get_bucket(self.bucket_name))
+        blob.upload_from_string(json.dumps(dataset_label))
+        
+        
+        
+
     
     
 # Download, Upload to GCS   
