@@ -1,19 +1,30 @@
 from datetime import datetime
-from enum import Enum
-# from kfp.v2.components.types.artifact_types import Artifact
+from enum import Enum, auto
+
+class GlobalVariable(): 
+    
+    class eCLOUD_TYPE(Enum):
+        GCP = auto()
+        AWS = auto()
+        AZURE = auto()
+        
+
+    class eDATASET_SOURCE_TYPE(Enum):
+        TENSRORFLOW = auto()
+        COCO = auto()
+
+        
+    class eDEEPLEARNING_FRAMEWORK_TYPE(Enum):
+        TENSRORFLOW = auto()
+        PYTORCH = auto()
 
 
-class eCLOUD_TYPE(Enum):
-    GCP = 1
-    AWS = 2
-    AZURE = 3
-
-class GlobalVariable():  
     def __new__(cls, *args, **kwargs):
         if not hasattr(cls, "_instance"):
             cls._instance = super().__new__(cls)  
         return cls._instance
     
+    
     def __init__(self, project, account):
         cls = type(self)
         if not hasattr(cls, "_init"):
@@ -26,6 +37,7 @@ class GlobalVariable():
             print(f"PROJECT_ID={self.PROJECT_ID}\nSERVICE_ACCOUNT_ID={self.SERVICE_ACCOUNT_ID}")
             print(f"COMPONENT_CONTAINER_URI={self.COMPONENT_CONTAINER_URI}\nTRAIN_CONTAINER_URI={self.TRAIN_CONTAINER_URI}\nSERVING_CONTAINER_URI={self.SERVING_CONTAINER_URI}")
     
+    
     VERSION = "0.0.3"
     PROJECT_ID = ""
     SERVICE_ACCOUNT_ID = ""
@@ -116,3 +128,13 @@ class GlobalVariable():
 
     # ##############
     BATCH_PREDICTION_FLAG = False
+    
+    
+    ################
+    # For TEST
+    LOCAL_EXEC = 1
+    PACKAGE_INSTALL_MODE = 0
+    DEV_MODE = 0 # 1 / 0
+    DEV_DATASET_METADATA_NAME = "metadata_dev.json" 
+    DEV_MODE_DATASET_SIZE = 100
+
